;;-------------------------------SLIDERS--------------------------
;;  population - total distinguish population in all planes
;;               ex: 2 planes with population 100 = 50 inicial population per plane
;;               note odd number of planes generade odd population so this can be divided equaly 
;;---------------------------------------------------------------- 


;;----------------------------GLOBALS-----------------------------------
;; y-plane n -  ycoordinates of plane n 
;; num-red/blue - number of red/blue turtles
;; n-cycles - number of cicles completed
;;
;; number-rewired - used to create small world networks
;; num-enc-cycle - number of encounters per cycle
;; 
;;----------------------------------------------------------------------
globals[
  sum-switch-ratio
  deadlocks-occurred 
  y-plane1 ;;y cordinates of plane1
  y-plane2 ;;y cordinates of plane2
  y-plane3 ;;y cordinates of plane3
  y-plane4 ;;y cordinates of plane4
  y-plane5 ;;y cordinates of plane5
  
  num-red  ;;number of red turtles
  num-blue ;; number of blue turtles
  
  n-cycles ;;number of cycles
  
  num-enc-cycle;;number of encounters performed in a cycle
  num-enc;;number of encounters performed
  number-rewired; used in small world network
  
  ;record the number of encounters by switching this cycle
  ;record the number of encounters by tolerance this cycle
  ;number of encounters not performed cycle
  total-num-switching-by-switching
  total-num-switching-by-tolerance
  total-num-switching-not-performed
  
  
  num-switching-by-switching-cycle
  num-switching-by-tolerance-cycle
  num-switching-not-performed-cycle
]

;------------------------------------TURTLE PROPRIETIES---------------------------------
;
;---------------------------------------------------------------------------------------
turtles-own[
  switched-last-step?
  num-switches
  memory
  mem-blue
  mem-red
  num-enc-red 
  num-enc-blue
  active? ;;specifies if the turtle is active or not
  network-links ;;list of lists of connections (who number): item 0 = connections in plane 1, 
              ;;item 1 connections on plane 2 ...and so on  
]

;---------------------------------------LINK PROPRIETIES----------------------------------
;
;-----------------------------------------------------------------------------------------
links-own[
  local-plane;;plane in which links are situated (usefull for manipulating link sets (small world creation)
  rewired?
]


;---------------------------------------SIMULATION SETUP--------------------------------
;
;---------------------------------------------------------------------------------------
to setup
  ;; (for this model to work with NetLogo's new plotting features,
  ;; __clear-all-and-reset-ticks should be replaced with clear-all at
  ;; the beginning of your setup procedure and reset-ticks at the end
  ;; of the procedure.)
  __clear-all-and-reset-ticks
  
  set-globals
  create-planes
  ;; turn half the turtles blue
  ask n-of (population / 2) turtles with [active? = true][set color blue]
  turtle-setup ;;setup turtle proprieties
  set-networks ;;creates the defined networks for plane 1 and plane 2 
end
;;--------------------------
;;set global initial values
;;--------------------------
to set-globals
  set deadlocks-occurred 0
  set sum-switch-ratio 0
  set y-plane1 -14
  set y-plane2 -7
  set y-plane3 0
  set y-plane4 7
  set y-plane5 14
  set num-red (population / num-planes)
  set num-blue (population / num-planes)
  
  set num-enc 0
  set num-enc-cycle 0
end
;;-----------------------------
;;setup turtle inicial values
;;-----------------------------
to turtle-setup
  ask turtles [
    set switched-last-step? false
    set num-switches 0
    set shape "circle"
    set size 0.6
    set network-links []
    set num-enc-red 0
    set num-enc-blue 0
    set memory []
    set mem-blue 0
    set mem-red 0  
  ]  
end


;----------------------------------------------------GO------------------------------------------
;
;------------------------------------------------------------------------------------------------
to go   
  if (stop-condition?)[
    update-globals
    stop
  ]
  ;;reset globals cycle clount 
  reset-cycle-stats
  
  if not (stop-condition?)[                             ;;if the % of consensus was not reached yet
    ask  turtles with [active? = true] [      ;;ask active turtles
        perform-encounter                            ;;perform an encounter with another active turtle 
        update-globals
    ]
    if (deadlocked?)[
      set deadlocks-occurred deadlocks-occurred + 1
      shake-agents
      ]
  ]
  
  ;;update number of encounters and number of cycles
  set num-enc (num-enc + num-enc-cycle)
  set n-cycles (n-cycles + 1)
  do-plots
  ;;end of a cycle
  tick
  
  
end

;reset the globals that record measures per cycle
to reset-cycle-stats
  set num-enc-cycle 0 
  set num-switching-by-switching-cycle 0
  set num-switching-by-tolerance-cycle 0
  set num-switching-not-performed-cycle 0
end

;;------------------------------------------------------------
;;                      STOP CONDITION
;;-------------------------------------------------------------
to-report stop-condition?
  report ( 
          (num-red  >= consensus-ratio * population) 
          or 
          (num-blue  >= consensus-ratio * population)
          or
          num-red = 0
          or 
          num-blue = 0
         )
end ;;end stop condition

;;-------------------------------------------------------------
;;                      UPDATE GLOBALS
;;-------------------------------------------------------------
to update-globals
  set num-red count turtles with [color = red ] ;;number of red turtles
  set num-blue count turtles with [color = blue ];; number of blue turtles
  set total-num-switching-by-switching total-num-switching-by-switching + num-switching-by-switching-cycle
  set total-num-switching-by-tolerance total-num-switching-by-tolerance + num-switching-by-tolerance-cycle
  set total-num-switching-not-performed total-num-switching-not-performed + num-switching-not-performed-cycle
end



to do-plots
  if do-plots? [
    set-current-plot "Consensus"
    set-current-plot-pen "red"
    plot count turtles with [color = red]
    set-current-plot-pen "blue"
    plot count turtles with [color = blue]
  
    set-current-plot "encounters"
    set-current-plot-pen "enc-cycle"
    plot num-enc-cycle
    set-current-plot-pen "avg-enc"
    plot num-enc / n-cycles
  ]
end







;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;
;;                                                agent encounter behaviour (called in turtle context)
;;
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
to perform-encounter  
  ;;get active neightbors
  let active-list get-active-who-list
  
  ;;if there is some neighbours to interact with
  if (length active-list > 0)[
      let partner turtle one-of active-list 
  
      ;;encounter and update color count information
      ifelse ([color] of partner = blue)
          [set num-enc-blue (num-enc-blue + 1)]
          [set num-enc-red (num-enc-red + 1)]
      
      if (num-enc-red > num-enc-blue)
        [set color red] 
      if (num-enc-blue > num-enc-red)
        [set color blue]
      ;;if the number of encounters is equal then mantain the same color
      set num-enc-cycle (num-enc-cycle + 1)
  ]
    
  ;; if there are other planes
  if (num-planes > 1)[
      ;;NEIGHBOURHOOD ANALISYS AND CONTEXT SWITCHING
      context-switch      
  ]
  
end;; end encounter behaviour

;;caled in turtle context
to-report get-good-neighbors-percentage
    let good-color color
    let current-plane ycor
    let active-neighbors get-active-who-list  
    let total-good 0
    
    foreach active-neighbors[
      if( [color] of turtle ? = good-color)
        [set total-good total-good + 1]
    ]
    
    ifelse( (length active-neighbors * 100) = 0 )[report 0]
         
    [report (total-good / length active-neighbors * 100)]
end


to context-switch
   let current-plane ycor
  
  
  let change false
  
  if(segregation?)[;; if segregation is enabled try to see if switching context is necessary
     ;;get percentage of good neighbors
     let good-neigbors (get-good-neighbors-percentage)
     let bad-neighbors (100 - good-neigbors)
   
     ;; 25% bad and high tolerance -> dont switch 
     ;; 75% bad and 50% tolerance  -> switch
     ;; 75% bad and low tolerance (30%) -> switch  
     if(current-plane = y-plane1)[
       if (bad-neighbors > plane-1-tolerance)[set change true]
     ]
     if(current-plane = y-plane2)[
       if (bad-neighbors > plane-2-tolerance)[set change true]
     ]
     if(current-plane = y-plane3)[
       if (bad-neighbors > plane-3-tolerance)[set change true]
     ]
     if(current-plane = y-plane4)[
       if (bad-neighbors > plane-4-tolerance)[set change true]
     ]
     if(current-plane = y-plane5)[
       if (bad-neighbors > plane-5-tolerance)[set change true]
     ]
  ]
  
    if(change = true)[
      switch
      set num-switching-by-tolerance-cycle num-switching-by-tolerance-cycle + 1
     ; show "switched by segregation"
      ] ;; switch by segregation directly
  
    if(not change and switching?)[ ;; if not changed by segregation and switching is allowed try to switch
       if(current-plane = y-plane1)[
         if (random 100 < plane-1-switching)[set change true]
       ]
       if(current-plane = y-plane2)[
         if (random 100 < plane-2-switching)[set change true]
       ]
       if(current-plane = y-plane3)[
         if (random 100 < plane-3-switching)[set change true]
       ]
       if(current-plane = y-plane4)[
         if (random 100 < plane-4-switching)[set change true]
       ]
       if(current-plane = y-plane5)[
         if (random 100 < plane-5-switching)[set change true]
       ]
  
       if( change = true)[
         switch
         ; show "switched by switching"
         set num-switching-by-switching-cycle num-switching-by-switching-cycle + 1  
       ]
        
      
    ]
    
      ;;flag a failled switching 
       if(change = false)[
         ;show "did not switch"
         set switched-last-step? false
         set num-switching-not-performed-cycle num-switching-not-performed-cycle + 1
         
         ]
end;;end context switch

to switch
  ;; this procedure is called in turtle context
  ;; so this variable are used to acess their atributes 
  let tcolor color
  let txcor xcor
  let current-plane ycor
  let tnum-enc-red num-enc-red 
  let tnum-enc-blue num-enc-red 
  let tnum-switches num-switches
  
  ;;turn current turtle inactive
    turn-inactive
  
    ;;select inactive clone at another plane
    let inactive-clone get-clone
  
    ;;activate clone
    ask inactive-clone [
        set color tcolor
        set num-enc-red  tnum-enc-red
        set num-enc-blue tnum-enc-red 
        set active? true  
        set num-switches (tnum-switches + 1) 
        set switched-last-step? true   
    ] 
end


to-report get-clone
  ;;select current turtle coordenates 
  let txcor xcor
  let tzcor zcor
  let tycor ycor
  
  ;;select clone
  let clone one-of turtles with [
                                  xcor = txcor and
                                  zcor = tzcor and
                                  ycor != tycor
                                 ]
  ;;return clone
  report clone 
end



;;------------------------------------------------------------------------------------
;; ACTIVE TURTLE IDS FROM LIST 
;;report a list of active turtle ids in "y-plane" from a list of turtles (who-list)
;;------------------------------------------------------------------------------------
to-report get-active-who-list 
  let current-plane ycor
  let context-neighbors link-neighbors
  let active-who-list []
  
  ask context-neighbors[
    if  [active?] of self = true and [ycor] of self = current-plane [
      set active-who-list (fput [who] of self active-who-list)
    ] 
  ]
  ;;report active id list
  report active-who-list
  
end

to turn-inactive 
  set color gray 
  set active? false
end

;;------------------------------------------------------------------------------------
;; INACTIVE TURTLE IDS FROM LIST 
;;report a list of inactive turtle ids in turtle current plane
;;------------------------------------------------------------------------------------
to-report get-inactive-who-list 
  let context-neighbors link-neighbors
  let current-plane ycor
   
  let inactive-who-list []
  
  ask context-neighbors[
    if  [active?] of self = false and [ycor] of self = current-plane [
      set inactive-who-list (fput [who] of self inactive-who-list)
    ] 
  ]
  ;;report active id list
  report inactive-who-list
  
end




;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;                                                 CREATE NETWORKS (CREATE NETWORKS OF AGENSTS IN EACH PLANE)
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------



;;---------------------------------------------------------------------------------
;; set the networks at all planes (num-planes)
;;---------------------------------------------------------------------------------
to set-networks
  let i 1
  repeat (num-planes)[
    create-network-at i 
    set i (i + 1)
  ]
end

;;---------------------------------------------------------------------------------
;;create a network at plane y-plane acording to interface chooser for that plane
;;---------------------------------------------------------------------------------
to create-network-at [plane-number]
   
      ifelse (get-network-by-plane-number (plane-number) = "regular") [create-regular k-regular (get-plane-by-number plane-number)][
        ifelse (get-network-by-plane-number (plane-number) = "scale-free") [create-scale-free (get-plane-by-number plane-number)]
          [create-random (get-plane-by-number plane-number)]
      ]
    
end
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;                                                 NETWORK CREATION 
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------

;;------------------------------------------------------------------
;;               CREATE RANDOM NETWORK AT A PLANE 
;;------------------------------------------------------------------
to create-random [plane]
 ;;create an inicial 2 nodes  and 1 edge
   let first-node one-of turtles with [ ycor = plane and color != grey ]
   let second-node one-of turtles with [ (self != first-node) and ycor = plane and color != grey] 
   
   add-network-link first-node second-node ;; make first edge (update network-links)
   
   
  ;;randomly select a new node
  let new-node one-of turtles with [ network-links = [] and ycor = plane and color != grey ]
 ;; and connect it to a partner already in the network
 let partner nobody
   while [ new-node != nobody ][
     set partner one-of turtles with [ (self != new-node) and ycor = plane and color != grey ]
     add-network-link new-node partner
     
     ;;update new node
     set new-node one-of turtles with [network-links = [] and ycor = plane]
   ]
end

;;------------------------------------------------------------------
;;               CREATE SCALE-FREE NETWORK AT A PLANE 
;;------------------------------------------------------------------
to create-scale-free [plane]
   ;;create an inicial 2 nodes  and 1 edge
   let first-node one-of turtles with [ ycor = plane and color != grey ]
   let second-node one-of turtles with [ self != first-node and ycor = plane and color != grey ] 
   
   ;;make first edge
   add-network-link first-node second-node
   
   
   ;;randomly select a new node
   let new-node one-of turtles with [network-links = [] and ycor = plane and color != grey ]
   
   ;;while there is a new-node to select
   let partner nobody
   while [ new-node != nobody ][
     set partner find-preferencial-partner (plane)
     add-network-link new-node partner
     
     ;;update new node
     set new-node one-of turtles with [ network-links = [] and ycor = plane ]
  
   ]
   
end

;;create regular network at plane "plane"
to create-regular  [k plane]
  ;; create turtle agent set for that plane
  let plane-turtles sort turtles with [ycor = plane]
 
  ;; iterate over the turtles
  let n 0
  while [n < length plane-turtles][
    ;; make edges with the next two neighbors
    let i 1
    while [i <= k ] [
      add-network-link (item n plane-turtles) 
                       (item ((n + i) mod length plane-turtles) plane-turtles) 
           
      set i i + 1             
    ]
    set n n + 1
   ]
end

;;------------------------------------------------------------------
;;               PREFERENCIAL ATTACHEMENT
;;------------------------------------------------------------------

to-report find-preferencial-partner [y-plane]
  let pick random-float sum [length network-links] of (turtles with [ycor = y-plane])
  let partner nobody
  ask turtles with [ycor = y-plane][
    if partner = nobody[ ;;if no turtle was picked yet
      ifelse length network-links > pick
          [set partner self]
          [set pick pick - (length network-links)]
    ]
  ]
  report partner
end


to remove-who-from [ r-who t-list ]
  set t-list remove-item (position r-who t-list) (t-list)
end



;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;                  AUX
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------


to add-network-link [node1 node2]
  ask node1 [set network-links fput ([who] of node2) network-links]
  ask node2 [set network-links fput ([who] of node1) network-links]
  ask node1 [create-link-with node2] 
  ask link ([who] of node1) ([who] of node2)[
    set local-plane ([pycor] of node1)
    
  ]
end

to wire-them [plane]
    ;; create turtle agent set for that plane
    let plane-turtles sort turtles with [ycor = plane]

  ;; iterate over the turtles
  let n 0
  while [n < length plane-turtles]
  [
    ;; make edges with the next two neighbors
    ;; this makes a lattice with average degree of 4
    make-edge-rewired (item n plane-turtles)
              (item ((n + 1) mod length plane-turtles) plane-turtles)
    make-edge-rewired (item n plane-turtles)
              (item ((n + 2) mod length plane-turtles) plane-turtles)
    set n n + 1
  ]
end

to make-edge-rewired [node1 node2]
  add-network-link node1 node2
  ask link ([who] of node1) ([who] of node2)[
    set rewired? false
  ] 
end




;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;                                                 CREATE PLANES (CREATE TURTLES AND INACTIVE CLONES IN EACH PLANE)
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------


;;------------------------------------------------------------------
;;                    CREATE N PLANES
;;------------------------------------------------------------------
to create-planes
  ;; create n planes according to "num-planes global"
  let i 1;
  repeat (num-planes)[
    create-plane i
    set i i + 1
  ]
end

;;------------------------------------------------------------------
;;                    CREATE A PLANE
;; recieves <- plane number
;;------------------------------------------------------------------
to create-plane [plane-number]
     ;;get the plane cor
     let y-plane get-plane-by-number (plane-number)
   
     ;;each random patch from y-plane create a active red turtle 
     ask n-of (population / num-planes) patches with [pycor = y-plane and (not any? turtles-here)][
        sprout 1[
          set color red
          set active? true
        ]  
     ] 
  ;;create inactive clones for y-plane on other planes
  create-inactives y-plane
end

;;---------------------------------------------------------------------------------
;;create inactive turtles coorresponding to the turtles of y-plane on other planes
;;---------------------------------------------------------------------------------
to create-inactives [o-y-plane]
ask turtles with [ycor = o-y-plane and color != grey][
      let txcor xcor  ;;xcor of the current turtle
      let tzcor zcor  ;;zcor of the current turtle
      let tid who     ;;who of the current turtle
      
      ;;create inactives
      let i 1
      repeat (num-planes)[
         if (get-plane-by-number (i) != (o-y-plane))[
           create-inactives-on-plane (get-plane-by-number i) txcor tzcor tid
         ]
         set i (i + 1) 
      ]
  ]
end

to create-inactives-on-plane [y-plane txcor tzcor tid]
    ;;ask patches with the same x z cor. of current turtle at another plane
      ask one-of patches with [ pycor = y-plane and 
                                pxcor = txcor and
                                pzcor = tzcor 
                              ]
       [
           sprout 1 [
             set color grey
             set active? false
           
       ]
    ]
end

;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;                                                                      AUX
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------
;;-------------------------------------------------------------------------------------------------------------------------------------------------------------------

;;report the y cor of a plane by its number e.g. 1 -> y-plane1
to-report get-plane-by-number [number]
let y-plane y-plane1;default plane 1
  ;; ------ which plane is it according to plane-number
  ifelse (number = 2)
    [set y-plane y-plane2]
    [ifelse (number = 3)
      [set y-plane y-plane3]
      [ifelse (number = 4)
        [set y-plane y-plane4]
        [if (number = 5) [set y-plane y-plane5]]
      ]
    ]
report y-plane
end

;;gets a network name for that plane
to-report get-network-by-plane-number [number]
  let network-name ""
  ifelse (number = 1) [set network-name (plane-1-network)]
    [ifelse (number = 2) [set network-name (plane-2-network)]
      [ifelse (number = 3) [set network-name (plane-3-network)]
        [ifelse (number = 4) [set network-name (plane-4-network)]
          [set network-name (plane-5-network)]
        ] 
      ]
     ]
  report network-name
end

to write-network
  file-open user-new-file
  let already_written []
  file-type "*Vertices "
  file-type population 
  file-type "\n"
  file-type "*arcs"
  file-type "\n"
   
  ask turtles with [ycor = y-plane1][
      
         set already_written fput who already_written
         let current who
         foreach network-links [
           if( not member? ? already_written )[
             let i current + 1
             let j ? + 1
       
             file-write i
             file-type ""  
             file-write j
             file-type "\n"
            ]
         ]       
       
  ]
 ; show sort already_written
  file-close
end

to-report deadlocked?
  let all-switched false
  ask turtles with [active? = true][
    set all-switched all-switched or switched-last-step?
    ]
    report not all-switched
end

to shake-agents
  let num-shaked 0
  ask turtles with [active? = true][
    if(random 2 = 1)[
      
      set num-shaked num-shaked + 1
      let tcolor color
      let txcor xcor
      let current-plane ycor
      let tnum-enc-red num-enc-red 
      let tnum-enc-blue num-enc-red 
      let tnum-switches num-switches
      
      turn-inactive
      
      ;;select inactive clone at another plane
      let inactive-clone get-clone
      
      ;;activate clone
      ask inactive-clone [
        set color tcolor
        set num-enc-red  tnum-enc-red
        set num-enc-blue tnum-enc-red 
        set active? true  
        set num-switches (tnum-switches + 1) 
        set switched-last-step? true   
      ] 
    ]
  ]
  show "SHAKED"
  show num-shaked
end

to-report get-num-deadlocks
  report deadlocks-occurred
end

to-report get-switch-ratio
  let total-switches 0
  ask turtles with [active? = true][
    if switched-last-step?
    [set total-switches total-switches + 1]]
  report total-switches / population
end

to-report global-switch-ratio
  set sum-switch-ratio sum-switch-ratio + get-switch-ratio
  report sum-switch-ratio / n-cycles
end
@#$#@#$#@
GRAPHICS-WINDOW
0
0
790
564
-1
20
13.0
1
10
1
1
1
0
1
1
1
0
59
-20
20
0
59
1
1
0
ticks
30.0

BUTTON
16
30
275
63
NIL
setup\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
16
100
275
133
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
16
135
275
168
population
population
0
3000
300
10
1
NIL
HORIZONTAL

CHOOSER
1182
35
1320
80
plane-1-network
plane-1-network
"regular" "scale-free" "random"
1

CHOOSER
1182
82
1320
127
plane-2-network
plane-2-network
"regular" "scale-free" "random"
1

SLIDER
1039
35
1179
68
plane-1-tolerance
plane-1-tolerance
0 
100 
70
1 
1
NIL
HORIZONTAL

SLIDER
1041
83
1180
116
plane-2-tolerance
plane-2-tolerance
0
100
30
1
1
NIL
HORIZONTAL

MONITOR
16
170
66
215
Red
count turtles with [[color] of self = red]
17
1
11

MONITOR
69
171
119
216
Blue
count turtles with [[color] of self = blue]
17
1
11

MONITOR
122
171
179
216
actives
count turtles with [[active?] of self = true]
17
1
11

PLOT
306
101
840
251
consensus
iterations
num agents
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""
"red" 1.0 0 -2674135 true "" ""
"blue" 1.0 0 -13345367 true "" ""

SWITCH
306
30
544
63
do-plots?
do-plots?
0
1
-1000

BUTTON
16
65
275
98
step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
181
171
275
216
cycles
n-cycles
17
1
11

SLIDER
1070
324
1242
357
k-regular
k-regular
0
150
150
1
1
NIL
HORIZONTAL

TEXTBOX
1072
301
1222
319
K Regular Parameter\n
10
0.0
1

SLIDER
1069
360
1241
393
num-planes
num-planes
1
5
2
1
1
NIL
HORIZONTAL

CHOOSER
1183
129
1321
174
plane-3-network
plane-3-network
"regular" "scale-free" "random"
0

CHOOSER
1184
178
1322
223
plane-4-network
plane-4-network
"regular" "scale-free" "random"
2

CHOOSER
1185
227
1323
272
plane-5-network
plane-5-network
"regular" "scale-free" "random"
0

SLIDER
1041
130
1181
163
plane-3-tolerance
plane-3-tolerance
0
100
20
1
1
NIL
HORIZONTAL

SLIDER
1042
179
1182
212
plane-4-tolerance
plane-4-tolerance
0
100
20
1
1
NIL
HORIZONTAL

SLIDER
1042
228
1183
261
plane-5-tolerance
plane-5-tolerance
0
100
24
1
1
NIL
HORIZONTAL

SLIDER
545
30
839
63
consensus-ratio
consensus-ratio
0.8
1
1
0.1
1
NIL
HORIZONTAL

MONITOR
16
218
119
263
encounters
num-enc
17
1
11

MONITOR
122
218
275
263
encounters per cycle
num-enc-cycle
17
1
11

PLOT
306
252
840
402
encounters
cycles
encounters
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""
"enc-cycle" 1.0 0 -955883 true "" ""
"avg-enc" 1.0 0 -13791810 true "" ""

SLIDER
857
35
1029
68
plane-1-switching
plane-1-switching
0
100
75
1
1
NIL
HORIZONTAL

SLIDER
858
82
1030
115
plane-2-switching
plane-2-switching
0
100
75
1
1
NIL
HORIZONTAL

SLIDER
858
130
1030
163
plane-3-switching
plane-3-switching
0
100
50
1
1
NIL
HORIZONTAL

SLIDER
857
177
1029
210
plane-4-switching
plane-4-switching
0
100
50
1
1
NIL
HORIZONTAL

SLIDER
859
218
1031
251
plane-5-switching
plane-5-switching
0
100
48
1
1
NIL
HORIZONTAL

SWITCH
306
65
544
98
switching?
switching?
1
1
-1000

MONITOR
16
266
275
311
DEADLOCKS
get-num-deadlocks
17
1
11

SWITCH
546
65
839
98
segregation?
segregation?
1
1
-1000

PLOT
305
403
841
621
Switching Dynamics
cycles
num-switching
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"switching-by-segregation" 1.0 0 -13791810 true "" "plot num-switching-by-switching-cycle"
"switching-by-switching" 1.0 0 -13840069 true "" "plot num-switching-by-tolerance-cycle"
"switchibg-not-performed" 1.0 0 -2674135 true "" "plot num-switching-not-performed-cycle"

@#$#@#$#@
## BASIC DESCRIPTION OF THE MODEL
In the context switching with segregation model, a society of agents engages in a simple
abstract game: the consensus game. Each agent has to choose towards one of two
possible choices which are basically arbitrary. The objective of the game is to
reach a global consensus, but the particular choice that gets collectively selected
is irrelevant. What is important is that overall agreement is achieved.
Agents can change their choice. Agents have the chance of changing the choice when they have an interaction with another agent in its neighbourhood (context), by playing the majority game: agents keep track of their previous interactions and select the choice that they have seen most often in the past. 

The agents are embedded in multiple relations represented as static social
networks. They switch context first by avoiding undersirable neighbourhoods by using a tolerance _T_ associated with each relation and second by a switching probability _S_ also associated with each social relation.

The agents are only active in one context at a time and can only perform encounters with available neighbours of the current context. We can think of context switching as a temporary deployment in another place, such as what happens with temporary immigration.


## AGENT BEHAVIOUR OVERVIEW ##
1. Compute the ratio _r_ of agents present in the current context _C<sub>i</sub>_ with a choice opposite from the current agent's choice.

2. If _r_ > _T<sub>C<sub>i</sub></sub>_(if the ratio is not within the social tolerance of the current
context):
	* switch to a randomly selected social context context _C<sub>j</sub>_

3. Otherwise:
	* switch to context _C<sub>j</sub>_ with a probability _S<sub>_C<sub>i</sub>_</sub>_ (the probability of switching from the context _C<sub>i</sub>_ to the selected context _C<sub>j</sub>_ when the social tolerance requirements fail).


## GUI BASIC EXPLANATION

**population:** the number of unique agents to be generated.
**switching?:** enables or disables the switching mechanism.
**segregation?:** enables or disables the segregation mechanism.

**note:** if switching is disabled the model uses a "shake" mechanism when the agents encounter a deadlock situation and increments the number of deadlocks.

**consensus-ratio:** the ration of agents in consensus necessary to the termination criteria.

**planet-n-switching:** parameter that determines the switching probability from the referred plane. This is used in the context switching mechanism.
**plane-n-tolerance:** parametar that determines the tolerance associated with a given 
plane n. This is used for the segregation mechanism. The agents only tolerate opinions different to their own up to this threshold.
**plane-n-network:** the network to be built in the referred plante. In this case the user can choose between an ER Network, Regular Network or a Scale-free network (Barabasi Albert Model)

**k-regular:** the parameter k for the construction of the k-regular networks.

**num-planes:** number of network layers to be created in the 3D CUBE.


## CREDITS AND REFERENCES

**Authors** 
*Davide Nunes*, *Luis Antunes*
GUESS / LabMAg / Universidade de Lisboa, Portugal
{davide.nunes, xaraxg@ di.fc.ul.pt}
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3D 5.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
